function [P, mle_B] = mre_generalize(a, r, A, R, B, objects_A, relations, objects_B)
%MRE_GENERALIZE Perform generalization for MRE
%
%   [P, mle_B] = mre_generalize(a, r, A, R, B, objects_A, relations,
%   objects_B)
%
% The function computes the MLE object b for object a and relation r (in 
% the original data representation), based on the trained MRE model that is
% specified in A, R, B, objects_A, relations, and objects_B. If the simple
% variant of MRE was used, B should be equal to A, and objects_B should be
% equal to objects_A.
%
%
% (C) Laurens van der Maaten, 2008
% Maastricht University


    % Retrieve desired object and relation
    Ai = A(:,:,objects_A == a);
    Ri = R(:,:,relations == r);
    
    % Compute pairwise similarity with each of the objects B
    P = squeeze(exp(-.5 * sum(sum(bsxfun(@minus, Ri * Ai, B) .^ 2, 1), 2)));
    P = P ./ sum(P);
    
    % Determine most likely object
    [mP, ind] = max(P);
    mle_B = objects_B(ind);
    