function [C, dC] = mre_grad(x, ind, nA, nR, nB, no_dims)
%MRE_GRAD Computes the gradient for the generic version of MRE
%
%   [C, dC] = mre_grad(x, ind, nA, nR, nB, no_dims)
%
% (C) Laurens van der Maaten, 2008
% Maastricht University


    % Decode solution
    A = reshape(x(1:nA * no_dims * no_dims), [no_dims no_dims nA]);
    R = reshape(x(1 + nA * no_dims * no_dims:(nA + nR) * no_dims * no_dims), [no_dims no_dims nR]);
    B = reshape(x(1 + (nA + nR) * no_dims * no_dims:end), [no_dims no_dims nB]);
    
    % Initialize gradients
    grad_A = zeros(no_dims, no_dims, nA);
    grad_R = zeros(no_dims, no_dims, nR);
    grad_B = zeros(no_dims, no_dims, nB);
    C = 0;

    % Loop over all given triples
    for i=1:size(ind, 1)

        % Get the current triple
        a = ind(i, 1);
        r = ind(i, 2);
        b = ind(i, 3);        
        Ai = A(:,:,a);
        Ri = R(:,:,r);
        Bi = B(:,:,b);

        % Compute the dot product of relation R and object A
        RA = Ri * Ai;

        % Compute probabilities
        P = exp(-.5 * sum(sum(bsxfun(@minus, RA, B) .^ 2, 1), 2));
        P = P ./ sum(P);

        % Expected value of object B under probabilities P
        eB = sum(bsxfun(@times, B, P), 3);        

        % Compute gradients
        grad_A(:,:,a) = grad_A(:,:,a) + Ri' * (eB - Bi);
        grad_R(:,:,r) = grad_R(:,:,r) + (eB - Bi) * Ai';
        grad_B(:,:,b) = grad_B(:,:,b) + Bi - RA;
        grad_B = grad_B + bsxfun(@times, P, bsxfun(@minus, RA, B));

        % Compute cost function
        C = C - log(P(b));
    end

    % Encode solution
    dC = [grad_A(:); grad_R(:); grad_B(:)];
    